const St = imports.gi.St;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
//const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;
const Util = imports.misc.util;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;

function CpuMemInfo(extensionMeta)
{
    this.init(extensionMeta);
}

CpuMemInfo.prototype =
{
    settings: null,
    paramTimerVal: null,
    paramUnitMem: null,
    paramEnableCpu: null,
    paramEnableMem: null,

    icon: null,
    labelCpu: null,
    labelUnitCpu: null,
    labelMem: null,
    labelUnitMem: null,
    box: null,
    button: null,

    timer: null,

    cpuTotalOld: null,
    cpuIdleOld: null,

    init: function(extensionMeta)
    {

    },


    onParamChanged: function()
    {
	this.paramTimerVal = this.settings.get_int('timer-val');
	if (this.timer != null)
            Mainloop.source_remove(this.timer);
	this.timer = Mainloop.timeout_add(this.paramTimerVal, Lang.bind(this, this.update));

	this.paramEnableCpu = this.settings.get_boolean('enable-cpu');
	if (this.paramEnableCpu)
	{
	    this.labelCpu.show();
	    this.labelUnitCpu.show();
	}
	else
	{
	    this.labelCpu.hide();
	    this.labelUnitCpu.hide();
	}

	this.paramEnableMem = this.settings.get_boolean('enable-mem');
	if (this.paramEnableMem)
	{
	    this.labelMem.show();
	    this.labelUnitMem.show();
	}
	else
	{
	    this.labelMem.hide();
	    this.labelUnitMem.hide();
	}

	this.paramUnitMem = this.settings.get_string('unit-mem');
	if (this.paramUnitMem != 'Go' && this.paramUnitMem != 'Mo' && this.paramUnitMem != '%')
	{
	    global.log(this.paramUnitMem);
	    this.settings.set_string('unit-mem', 'Go');
	    return;
	}
	this.labelUnitMem.set_text(this.paramUnitMem);

	this.update();
    },
    
    enable: function()
    {
        this.icon = new St.Icon(
        {
//	    gicon: Gio.icon_new_for_string(meta.path + "/processor.png"),
//	    icon_type: St.IconType.SYMBOLIC,
	    icon_name: "utilities-system-monitor-symbolic",
	    icon_size: 16,
	    style_class: 'cmi-icon'
        });

	this.labelCpu = new St.Label({
	    style_class: 'cmi-cpu-label'
	});

	this.labelUnitCpu = new St.Label({
	    text: '%',
	    style_class: 'cmi-cpu-unit-label'
	});

	this.labelMem = new St.Label({
	    style_class: 'cmi-mem-label'
	});

	this.labelUnitMem = new St.Label({
	    style_class: 'cmi-mem-unit-label'
	});

	this.box = new St.BoxLayout();
        this.box.add_actor(this.icon);
        this.box.add_actor(this.labelCpu);
        this.box.add_actor(this.labelUnitCpu);
        this.box.add_actor(this.labelMem);
        this.box.add_actor(this.labelUnitMem);

	this.button = new St.Button({
	    style_class: 'panel-button'
	});
        this.button.connect('clicked', this.clicked);
        this.button.set_child(this.box);

	Main.panel._rightBox.insert_child_at_index(this.button, 0);

	this.settings = Lib.getSettings('org.gnome.shell.extensions.CpuMemInfo');
//	this.settings.connect("changed::timer-val", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-cpu", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-mem", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::unit-mem", Lang.bind(this, this.onParamChanged));

	this.onParamChanged();
    },

    disable: function()
    {
	Main.panel._rightBox.remove_child(this.button);

        Mainloop.source_remove(this.timer);
    },

    clicked: function()
    {
	let app = global.log(Shell.AppSystem.get_default().lookup_app('gnome-system-monitor.desktop'));

        if (app != null)
            app.activate();
        else
            Util.spawn(['gnome-system-monitor']);
    },

    update: function()
    {
	if (this.paramEnableCpu)
        {
	    let cpuTotal = 0, cpuIdle;

	    let fileStat = GLib.file_get_contents('/proc/stat');
            linesStat = ("" + fileStat[1]).split("\n");

            for (let i = 0; i < linesStat.length; i++)
	    {
		let values;

		if (linesStat[i].match(/^cpu /))
		{
                    values = linesStat[i].match(/^cpu\s*(.*)$/)[1].split(' ');

		    cpuIdle = values[3];

		    for (let i = 0; i < values.length; i++)
			cpuTotal += parseInt(values[i]);
		}
	    }

	    if (typeof(this.cpuTotalOld) == "undefined" && typeof(this.cpuIdleOld) == "undefined")
	    {
		this.cpuTotalOld = cpuTotal - 1;
		this.cpuIdleOld = cpuIdle - 1;
	    }

	    let diffCpuTotal = cpuTotal - this.cpuTotalOld;
	    let diffCpuIdle = cpuIdle - this.cpuIdleOld;

	    let cpuValue = Math.round(100 * (diffCpuTotal - diffCpuIdle) / diffCpuTotal);

	    this.cpuTotalOld = cpuTotal;
	    this.cpuIdleOld = cpuIdle;

            this.labelCpu.set_text(cpuValue.toString());
	}

	if (this.paramEnableMem)
	{
       	    let memTotal, memFree, memBuffers, memCached;

	    let fileMeminfo = GLib.file_get_contents('/proc/meminfo');
	    linesMeminfo = ("" + fileMeminfo[1]).split("\n");

	    for (let i = 0; i < linesMeminfo.length; i++)
	    {
		let values;

		if (linesMeminfo[i].match(/^MemTotal/))
		{
		    values = linesMeminfo[i].match(/^MemTotal:\s*([^ ]*)\s*([^ ]*)$/);
		    
		    memTotal = values[1];
		}
		else if (linesMeminfo[i].match(/^MemFree/))
		{
		    values = linesMeminfo[i].match(/^MemFree:\s*([^ ]*)\s*([^ ]*)$/);
		    
		    memFree = values[1];
		}
		else if (linesMeminfo[i].match(/^Buffers/))
		{
		    values = linesMeminfo[i].match(/^Buffers:\s*([^ ]*)\s*([^ ]*)$/);
		    
		    memBuffers = values[1];
		}
		else if (linesMeminfo[i].match(/^Cached/))
		{
		    values = linesMeminfo[i].match(/^Cached:\s*([^ ]*)\s*([^ ]*)$/);
		    
		    memCached = values[1];
		}	    
            }

	    let memUsed = memTotal - memFree - memBuffers - memCached;

	    if (this.paramUnitMem == "%")
		this.labelMem.set_text(Math.round(memUsed / memTotal * 100).toString());
	    else
	    {
		let precision = 1;
		let unitFactor = 1024;

		if (this.paramUnitMem == "Go")
		{
		    unitFactor = 1048576;

		    if (memTotal >= 10 * unitFactor)
			precision = 10;
		    else
			precision = 100;
		}

		memUsed = Math.round(precision * memUsed / unitFactor) / precision;
		memTotal = Math.round(precision * memTotal / unitFactor) / precision;

		this.labelMem.set_text(memUsed.toString() + " / " + memTotal.toString());
	    }
	}

	return true;
    }
}

function init(extensionMeta)
{
    return new CpuMemInfo(extensionMeta);
}
