const Gtk = imports.gi.Gtk;

const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;

let settings;

function init()
{
    settings = Lib.getSettings('org.gnome.shell.extensions.CpuMemInfo');
}

function buildPrefsWidget()
{
    let frame = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10 });
    let label = new Gtk.Label({ label: "<b>Global</b>", use_markup: true, xalign: 0 });
    let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
                             margin_left: 20 });



    let hboxTimerVal = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
    let labelTimerVal = new Gtk.Label({label: "Update Interval (seconds) ", xalign: 0});
    labelTimerVal.set_tooltip_text("Need restart extension");
    let valueTimerVal = new Gtk.Adjustment({
        lower: 1,
        upper: 300,
        step_increment: 1
    });
    let value2TimerVal = new Gtk.HScale({
        digits: 0,
        adjustment: valueTimerVal,
        value_pos: Gtk.PositionType.RIGHT
    });
    value2TimerVal.set_value(Math.round(settings.get_int("timer-val") / 1000));
    valueTimerVal.connect('value-changed',
			  function(sw)
			  {
			      settings.set_int("timer-val", sw.get_value() * 1000);
			  });
    hboxTimerVal.add(labelTimerVal);
    hboxTimerVal.pack_end(value2TimerVal, true, true, 0);
    vbox.add(hboxTimerVal);


    let hboxEnableCpu = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
    let labelEnableCpu = new Gtk.Label({label: "Enable Cpu", xalign: 0});
    let valueEnableCpu = new Gtk.Switch({active: settings.get_boolean("enable-cpu")});
    valueEnableCpu.connect('notify::active',
			   function(button)
			   {
			       settings.set_boolean("enable-cpu", button.active);
			   });
    hboxEnableCpu.pack_start(labelEnableCpu, true, true, 0);
    hboxEnableCpu.add(valueEnableCpu);
    vbox.add(hboxEnableCpu);


    let hboxEnableMem = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
    let labelEnableMem = new Gtk.Label({label: "Enable Mem", xalign: 0});
    let valueEnableMem = new Gtk.Switch({active: settings.get_boolean("enable-mem")});
    valueEnableMem.connect('notify::active',
			   function(button)
			   {
			       settings.set_boolean("enable-mem", button.active);
			   });
    hboxEnableMem.pack_start(labelEnableMem, true, true, 0);
    hboxEnableMem.add(valueEnableMem);
    vbox.add(hboxEnableMem);


    let hboxUnitMem = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
    let labelUnitMem = new Gtk.Label({label: "Unit Mem", xalign: 0});
    let value1UnitMem = new Gtk.RadioButton({label: "Go"});
    let value2UnitMem = new Gtk.RadioButton({label: "Mo", group: value1UnitMem});
    let value3UnitMem = new Gtk.RadioButton({label: "%", group: value1UnitMem});
    if (settings.get_string("unit-mem") == 'Go')
	value1UnitMem.set_active(true);
    else if (settings.get_string("unit-mem") == 'Mo')
	value2UnitMem.set_active(true);
    else
	value3UnitMem.set_active(true);
    value1UnitMem.connect('toggled',
			  function(button)
			  {
			      if (button.get_active())
				  settings.set_string("unit-mem", "Go");
			  });
    value2UnitMem.connect('toggled',
			  function(button)
			  {
			      if (button.get_active())
				  settings.set_string("unit-mem", "Mo");
			  });
    value3UnitMem.connect('toggled',
			  function(button)
			  {
			      if (button.get_active())
				  settings.set_string("unit-mem", "%");
			  });
    hboxUnitMem.pack_start(labelUnitMem, true, true, 0);
    hboxUnitMem.add(value1UnitMem);
    hboxUnitMem.add(value2UnitMem);
    hboxUnitMem.add(value3UnitMem);
    vbox.add(hboxUnitMem);

    frame.add(label);
    frame.add(vbox);
    frame.show_all();

    return frame;
}
